import json

__author__ = 'Yaisel Hurtado <hurta2yaisel@gmail.com>'
__date__ = '15/06/18'


class Event(object):
    def __init__(self):
        self.TimeStamp = None
        self.EventDescription = None
        self.Dimensions = {}
        self.Properties = {}
        self.EventType = "APIRequest"

    @property
    def event_type(self):
        return self.EventType

    def to_json(self):
        return json.dumps(self.__dict__)


class DependencyEvent(Event):
    def __init__(self, dependency_name, dependency_duration):
        super(DependencyEvent, self).__init__()
        self.Properties.update({
            "DependencyName": dependency_name,
            "DependencyDuration": dependency_duration,
        })
        self.EventType = "Dependency"

    @property
    def dependency_name(self):
        if "DependencyName" in self.Dimensions:
            return str(self.Properties.get('DependencyName', ''))
        return ''

    @dependency_name.setter
    def dependency_name(self, value):
        self.Properties["DependencyName"] = str(value)

    @property
    def dependency_duration(self):
        if "DependencyDuration" in self.Dimensions:
            return int(str(self.Properties.get('DependencyDuration', -1)))
        return -1

    @dependency_duration.setter
    def dependency_duration(self, value):
        self.Properties["DependencyDuration"] = int(value)


class ExceptionEvent(Event):
    def __init__(
            self, exception_message, exception_type, exception_stack_trace
    ):
        super(ExceptionEvent, self).__init__()
        self.Properties.update({
            "ExceptionMessage": exception_message,
            "ExceptionType": exception_type,
            "ExceptionStackTrace": exception_stack_trace
        })
        self.EventType = "Exception"

    @property
    def exception_message(self):
        if "ExceptionMessage" in self.Dimensions:
            return str(self.Properties.get('ExceptionMessage', ''))
        return ''

    @exception_message.setter
    def exception_message(self, value):
        self.Properties["ExceptionMessage"] = str(value)

    @property
    def exception_type(self):
        if "ExceptionType" in self.Dimensions:
            return int(str(self.Properties.get('ExceptionType', '')))
        return ''

    @exception_type.setter
    def exception_type(self, value):
        self.Properties["ExceptionType"] = str(value)

    @property
    def exception_stack_trace(self):
        if "ExceptionStackTrace" in self.Dimensions:
            return int(str(self.Properties.get('ExceptionStackTrace', '')))
        return ''

    @exception_stack_trace.setter
    def exception_stack_trace(self, value):
        self.Properties["ExceptionStackTrace"] = str(value)



#!-*- coding: utf-8 -*-

from itertools import cycle
from random import randint

import pytest
from faker import Faker

fake = Faker()


def calculate_dv(rut):
    reversed_digits = map(int, reversed(str(rut)))
    factors = cycle(range(2, 8))
    s = sum(d * f for d, f in zip(reversed_digits, factors))
    dv = (-s) % 11
    if dv == 10:
        return 'K'
    return dv


def generate_rut():
    rut = randint(10000000, 99999999)
    return '{rut}-{dv}'.format(rut=rut, dv=calculate_dv(rut))


@pytest.fixture(scope='function')
def customer_kwargs():
    customer_id = generate_rut()
    return dict(
        customer_id=customer_id,
        customer_code=fake.ssn(),
        name=fake.company(),
        address=fake.street_address(),
        city=fake.city(),
        zip_code=fake.postcode(),
        website=fake.url()
    )



import os

from fc.cloudtrailsdk.client.tracker import Tracker


def configure_tracker():
    app_name = os.environ.get("AWS_LAMBDA_FUNCTION_NAME", "fc-notify")
    app_version = os.environ.get("APP_VERSION")
    region = os.environ.get("AWS_REGION")
    tracker_environment = os.environ.get("TRACKER_ENVIRONMENT","eCloudTrailsStreamQA")
    tracker = Tracker(
        tracker_environment,
        region=region,
        app_name=app_name,
        app_version=app_version
    )
    return tracker


import os
import json
from fc.cloudtrailsdk.client.tracker import Tracker
from fc.cloudtrailsdk.model.event import DependencyEvent, Event, ExceptionEvent
from fc.cloudtrailsdk.utils.functions import configure_tracker

class CustomLogger:
    def __init__(self):
        self.tracker = configure_tracker()

    def __call__(self, func): return lambda *args, **kwargs: self.callFunc(func, *args, **kwargs)

    def callFunc(self, func, *args, **kwargs):
        func_name=func.__name__
        r = func(*args, **kwargs)
        custom_event = Event()
        custom_event.Properties.update({
            'Method': func_name,
            'RequestPayload': json.dumps(kwargs) if len(kwargs)>0 else json.dumps(args),
            'ResponsePayload': json.dumps(r),
            'ResponseHttpStatus': 200
        })
        self.tracker.track_event(custom_event)
        return r


from __future__ import absolute_import, unicode_literals

import datetime
import logging

import boto3
import sys

__author__ = 'Yaisel Hurtado <hurta2yaisel@gmail.com>'
__date__ = '15/06/18'

logger = logging.getLogger(__name__)


class Tracker(object):
    def __init__(
            self, stream_name, credentials=None, region=None,
            app_name=None, app_version=None
    ):
        if not credentials:
            credentials = {}

        self.firehose_client = boto3.client(
            'firehose',
            region_name=region,
            aws_access_key_id=credentials.get('aws_access_key_id', None),
            aws_secret_access_key=credentials.get('aws_secret_access_key', None)
        )

        self.delivery_stream_name = stream_name
        self.dimensions = {}
        self.app_name = app_name
        self.app_version = app_version

    def track_event(self, track_event):
        return self.__internal_track_event(track_event)

    def track_exception(self, exception_event):
        return self.__internal_track_event(exception_event)

    def track_dependency(self, dependency_event):
        return self.__internal_track_event(dependency_event)

    def __internal_track_event(self, track_event):
        try:
            track_event.Dimensions.update(self.dimensions)
            timestamp = int((datetime.datetime.utcnow() - datetime.datetime(1970, 1, 1)).total_seconds())
            track_event.TimeStamp = str(timestamp)
            track_event.Dimensions.update({
                "dApplication": self.app_name,
                "dApplicationVersion": self.app_version
            })

            success = False
            max_tries = 3
            data = track_event.to_json()
            if (sys.version_info > (3, 0)):
                data = bytes(data.encode('utf-8'))
            else:
                data = bytes(data)
            record = {
                'Data': data
            }

            while not success and max_tries > 0:
                try:
                    response = self.firehose_client.put_record(
                        DeliveryStreamName=self.delivery_stream_name,
                        Record=record
                    )
                    success = True if response else False

                except Exception as e:
                    max_tries -= 1
                    logger.exception(e.message, *e.args)
                    logger.info('Retrying...')
                    continue

            return success

        except Exception as e:
            logger.exception(e.message, *e.args)
            return False



from functools import wraps

import sys


from fc.cloudtrailsdk.model.event import ExceptionEvent
from fc.cloudtrailsdk.utils.functions import configure_tracker


def exception_logger(*params_args, **params_kwargs):
    def decorator(f):
        @wraps(f)
        def wrapper(*args, **kwargs):

            try:
                response = f(*args, **kwargs)
                return response
            except Exception as e:
                tracker = configure_tracker()
                exception_event = ExceptionEvent(*sys.exc_info())
                tracker.track_exception(exception_event)
                raise
        return wrapper
    return decorator



import json

from faker import Faker

from fc.cloudtrailsdk.client.tracker import Tracker
from fc.cloudtrailsdk.decorators import custom_logger
from fc.cloudtrailsdk.model.event import DependencyEvent, Event, ExceptionEvent
from fc.cloudtrailsdk.decorators import exception_logger
from tests.conftest import customer_kwargs

__author__ = 'Yaisel Hurtado <hurta2yaisel@gmail.com>'
__date__ = '15/06/18'

fake = Faker()


class TestTracker(object):
    pass


def main():
    try:
        credentials = {
            'aws_access_key_id': "AKIAJRTLLEJZH4IHID7Q",
            'aws_secret_access_key': "nnJgGu8BUMBoDTjNmNBhe1jqk9FFWJhdpGbIkfsj"
        }
        tracker = Tracker(
            "eCloudTrailsStreamQA",
            credentials=credentials,
            region='us-east-1',
            app_name=fake.name(),
            app_version=fake.building_number()
        )
        customer = customer_kwargs()
        tracker.dimensions.update({
            'Customer': customer['name']
        })

        custom_event = Event()
        custom_event.Properties.update({
            'Method': 'Register',
            'RequestPayload': json.dumps(customer_kwargs()),
            'ResponsePayload': json.dumps({"status": "ok"}),
            'ResponseHttpStatus': 200
        })
        tracker.track_event(custom_event)

        print("Tracked Event")

        dependency_event = DependencyEvent("webhook01", 3000)
        tracker.track_dependency(dependency_event)
        print("Tracked Dependency")

        exception_event = ExceptionEvent(
            "Mensaje de error", "Tipo de Excepcion", "El stacktrace"
        )
        tracker.track_exception(exception_event)
        print("Tracked Exception")
        print("Done!")
    except Exception as e:
        print("Error: %s" % e)

@custom_logger()
def test_exception_logger(url= "http://localhost"):
    print("success")
if __name__ == '__main__':
    test_exception_logger()
    # main()




import os

import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()


VERSION = os.environ.get("TAG_VERSION")
VERSION = VERSION[2:] if VERSION else "0.0.15"

setuptools.setup(
    name="CloudTrails SDK",
    version=VERSION,
    author="Yaisel Hurtado, Raydel Miranda",
    author_email="hurta2yaisel@gmail.com, raydel.miranda.gomez@gmail.com",
    description="SDK for CloudTrails DevOps management",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/elasbit/fc-cloudtrails-sdk-py",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 2",
        "Operating System :: OS Independent",
    ),
)
